/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.storage.eclipse;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;

/**
 * An iterator class which is similar to the iterator returned from
 * {@link EObject#eAllContents()}, but can be used on multiple objects and also
 * includes the objects themselves.
 * 
 * @author hummelb
 * @author $Author$
 * @version $Rev$
 * @levd.rating GREEN Rev: 1794
 */
public class EMultiContentsIterator implements Iterator<EObject> {

	/** The objects to iterate on. */
	private final EObject[] objects;

	/** Current index into the array. */
	private int currentIndex = 0;

	/** Inner iterator. */
	private Iterator<EObject> inner = null;

	/** Constructor. */
	public EMultiContentsIterator(EObject... objects) {
		this.objects = objects;
	}

	/** Constructor. */
	public EMultiContentsIterator(Collection<EObject> objects) {
		this.objects = objects.toArray(new EObject[objects.size()]);
	}

	/** {@inheritDoc} */
	@Override
	public boolean hasNext() {
		return currentIndex < objects.length;
	}

	/** {@inheritDoc} */
	@Override
	public EObject next() {
		EObject result;
		if (inner == null) {
			result = objects[currentIndex];
			inner = result.eAllContents();
		} else {
			result = inner.next();
		}

		// already check whether there is more to come
		if (!inner.hasNext()) {
			inner = null;
			currentIndex++;
		}

		return result;
	}

	/** Removal is not supported. */
	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}
}
